package com.kdm.erraiexample.client;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;

import org.jboss.errai.databinding.client.api.DataBinder;
import org.jboss.errai.databinding.client.api.PropertyChangeEvent;
import org.jboss.errai.databinding.client.api.PropertyChangeHandler;
import org.jboss.errai.ui.client.widget.HasModel;
import org.jboss.errai.ui.client.widget.ListWidget;
import org.jboss.errai.ui.shared.api.annotations.AutoBound;
import org.jboss.errai.ui.shared.api.annotations.Bound;
import org.jboss.errai.ui.shared.api.annotations.DataField;
import org.jboss.errai.ui.shared.api.annotations.EventHandler;
import org.jboss.errai.ui.shared.api.annotations.Templated;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.Composite;
import com.kdm.erraiexample.client.model.ChildModel;
import com.kdm.erraiexample.client.model.ParentList;
import com.kdm.erraiexample.client.model.ParentModel;
import com.kdm.erraiexample.client.widget.ParentWidget;

@ApplicationScoped
@Templated("#main")
public class TopLevelComponent extends Composite implements HasModel<ParentList>
{
  private static final Logger log = LoggerFactory.getLogger(TopLevelComponent.class);

  @Inject
  @AutoBound
  private DataBinder<ParentList> parentListBinder;

  @Inject
  @Bound(property = "parents")
  @DataField
  private ListWidget<ParentModel, ParentWidget> parentList;

  @Inject
  @Bound(property = "test")
  @DataField
  private CheckBox test;

  @PostConstruct
  private void initInstance()
  {
    log.info("LeftPanelComponent init()..");

    List<ParentModel> parents = createParentList();
    parentListBinder.getModel().setParents(parents);

    parentListBinder.addPropertyChangeHandler("**", new PropertyChangeHandler<Object>()
    {
      @Override
      public void onPropertyChange(PropertyChangeEvent<Object> event)
      {
        String propertyName = event.getPropertyName();
        Object newValue = event.getNewValue();
        log.info("  onPropertyChange: " + propertyName + " val ='" + newValue + " '");
      }
    });
  }

  private List<ParentModel> createParentList()
  {
    ParentModel parent1 = new ParentModel();
    parent1.setParentValue("parent 1!");
    List<ChildModel> children = new ArrayList<>();
    children.add(new ChildModel("test1"));
    children.add(new ChildModel("test2"));
    children.add(new ChildModel("test3"));
    children.add(new ChildModel("test4"));
    parent1.setChildren(children);

    ParentModel parent2 = new ParentModel();
    parent2.setParentValue("parent 2!");
    children = new ArrayList<>();
    children.add(new ChildModel("test5"));
    children.add(new ChildModel("test6"));
    children.add(new ChildModel("test7"));
    children.add(new ChildModel("test8"));
    parent2.setChildren(children);

    List<ParentModel> parents = new ArrayList<>();
    parents.add(parent1);
    parents.add(parent2);
    return parents;
  }

  public ListWidget<ParentModel, ParentWidget> getParentList()
  {
    return parentList;
  }

  public void setParentList(ListWidget<ParentModel, ParentWidget> parentList)
  {
    this.parentList = parentList;

  }

  @Override
  public ParentList getModel()
  {
    return parentListBinder.getModel();
  }

  @Override
  public void setModel(ParentList model)
  {
    parentListBinder.setModel(model);
  }

  @EventHandler("test")
  private void withTest(ClickEvent event)
  {
    log.info("test: should be triggering something?");
    // Shouldn't clicking on the bound value of this model cause a Property
    // Change Event to occur? Isn't it changing the value of that model?

    // uIFacetListBinder.getModel().getEntries().get(0).setCount(99999);
  }
}
