package com.kdm.erraiexample.client.widget;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.jboss.errai.databinding.client.api.DataBinder;
import org.jboss.errai.databinding.client.api.PropertyChangeEvent;
import org.jboss.errai.databinding.client.api.PropertyChangeHandler;
import org.jboss.errai.ui.client.widget.HasModel;
import org.jboss.errai.ui.client.widget.ListWidget;
import org.jboss.errai.ui.shared.api.annotations.AutoBound;
import org.jboss.errai.ui.shared.api.annotations.Bound;
import org.jboss.errai.ui.shared.api.annotations.DataField;
import org.jboss.errai.ui.shared.api.annotations.EventHandler;
import org.jboss.errai.ui.shared.api.annotations.Templated;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.InlineLabel;
import com.kdm.erraiexample.client.model.ChildModel;
import com.kdm.erraiexample.client.model.ParentModel;

@Templated("#parent")
public class ParentWidget extends Composite implements HasModel<ParentModel>
{
  private static final Logger log = LoggerFactory.getLogger(ParentWidget.class);

  @Inject
  @AutoBound
  private DataBinder<ParentModel> parentBinder;

  @Inject
  @Bound(property = "parentValue")
  @DataField
  private InlineLabel parentValue;

  @Inject
  @Bound(property = "test")
  @DataField
  private CheckBox test;
    
  @Inject
  @Bound(property = "children")
  @DataField
  private ListWidget<ChildModel, ChildWidget> childList;

  @PostConstruct
  public void initInstance()
  {
    parentBinder.addPropertyChangeHandler("**",new PropertyChangeHandler<Object>()
    {
      @Override
      public void onPropertyChange(PropertyChangeEvent<Object> event)
      {
        String propertyName = event.getPropertyName();
        Object newValue = event.getNewValue();
        log.info(" parentBinder onPropertyChange: " + propertyName + " val ='"
            + newValue + " '");
      }
    });
  }

  @Override
  public ParentModel getModel()
  {
    return parentBinder.getModel();
  }

  @Override
  public void setModel(ParentModel model)
  {
    parentBinder.setModel(model);
  }
  
  @EventHandler("test")
  private void withTest(ClickEvent event)
  {
    // Should THIS trigger the value change callbacK?
    boolean checked = ((CheckBox) event.getSource()).getValue();
    log.info("--- Selected=" + checked+"  but this should cause a property change event visible to topLevelComponent?");

    // uncommenting this: does not work either. it's modifying values explicitly;
    // should this not cause Property change event visible to the topLevel Component
    // because it's under the FacetUIList graph?     
    //    facetUIBinder.getModel().getEntries().get(0).setCount(99999);
    

  }
  
}
